@@ -26,14 +26,17 @@ class FederatedInterfaceType(InterfaceType):
         if callable(self._reference_resolver):
             graphql_type = schema.type_map.get(self.name)
             setattr(
-                graphql_type, "__resolve_reference__", self._reference_resolver,
+                graphql_type,
+                "__resolve_reference__",
+                self._reference_resolver,
             )
 
             for object_type in schema.type_map.values():
-                if _type_implements_interface(self.name, object_type):
-                    if not hasattr(object_type, "__resolve_reference__"):
-                        setattr(
-                            object_type,
-                            "__resolve_reference__",
-                            self._reference_resolver,
-                        )
+                if _type_implements_interface(self.name, object_type) and not hasattr(
+                    object_type, "__resolve_reference__"
+                ):
+                    setattr(
+                        object_type,
+                        "__resolve_reference__",
+                        self._reference_resolver,
+                    )
