@@ -563,7 +563,9 @@ def heal_schema(schema: GraphQLSchema) -> GraphQLSchema:
 
         each(type_.fields, _heal_field)
 
-    def heal_type(type_: GraphQLNamedType) -> GraphQLNamedType:
+    def heal_type(
+        type_: Union[GraphQLList, GraphQLNamedType, GraphQLNonNull]
+    ) -> Union[GraphQLList, GraphQLNamedType, GraphQLNonNull]:
         # Unwrap the two known wrapper types
         if isinstance(type_, GraphQLList):
             type_ = GraphQLList(heal_type(type_.of_type))
